package se.gawell.setrace.logging;

import java.lang.reflect.Field;
import java.util.Collection;
import java.util.Date;

public class SimpleToStringGenerator implements ToStringGenerator {

	public void generateToString(StringBuilder stringBuilder, Object object) {
		if (object instanceof String) {
			stringBuilder.append("\"").append(object).append("\"");
			return;
		}

		if (object instanceof Collection) {
			@SuppressWarnings("rawtypes")
			Collection collection = (Collection) object;
			stringBuilder.append("{").append(collection.size())
					.append(" elements}");
			return;
		}

		if (shouldBeAppendedDirrectly(object)) {
			stringBuilder.append(object);
			return;
		}

		try {
			boolean firstFieldWritten = false;
			for (Field field : object.getClass().getFields()) {
				if (firstFieldWritten) {
					stringBuilder.append(" ");
				}
				firstFieldWritten = true;
				stringBuilder.append(field.getName()).append("=");
				stringBuilder.append("[");
				generateToString(stringBuilder, field.get(object));
				stringBuilder.append("]");
			}
		} catch (IllegalArgumentException e) {
			throw new RuntimeException(e);
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		}
	}

	private boolean shouldBeAppendedDirrectly(Object object) {
		if (object == null)
			return true;
		if (object.getClass().isPrimitive())
			return true;
		if (object.getClass().isAssignableFrom(Integer.class))
			return true;

		if (object.getClass().isAssignableFrom(Long.class))
			return true;

		if (object.getClass().isAssignableFrom(Boolean.class))
			return true;

		if (object.getClass().isAssignableFrom(Character.class))
			return true;

		if (object.getClass().isAssignableFrom(Date.class))
			return true;

		if (object.getClass().isEnum())
			return true;

		return false;
	}

}
